# Generated by roxygen2: do not edit by hand

export(A_theta_x)
export(F_from_f)
export(F_inv_from_f)
export(Gauss_Legen_nodes)
export(Gauss_Legen_weights)
export(Gegen_coefs)
export(Gegen_coefs_2d)
export(Gegen_coefs_Pn)
export(Gegen_norm)
export(Gegen_norm_2d)
export(Gegen_polyn)
export(Gegen_polyn_2d)
export(Gegen_series)
export(Gegen_series_2d)
export(H_ab)
export(Psi_mat)
export(Theta_to_X)
export(X_to_Theta)
export(akx)
export(angles_to_sphere)
export(avail_cir_tests)
export(avail_sph_tests)
export(bk_to_uk)
export(bk_to_vk2)
export(c_unif_cap)
export(cir_gaps)
export(cir_stat_Ajne)
export(cir_stat_Bakshaev)
export(cir_stat_Bingham)
export(cir_stat_CCF09)
export(cir_stat_Cressie)
export(cir_stat_FG01)
export(cir_stat_Gine_Fn)
export(cir_stat_Gine_Gn)
export(cir_stat_Gini)
export(cir_stat_Gini_squared)
export(cir_stat_Greenwood)
export(cir_stat_Hermans_Rasson)
export(cir_stat_Hodges_Ajne)
export(cir_stat_Kuiper)
export(cir_stat_Log_gaps)
export(cir_stat_Max_uncover)
export(cir_stat_Num_uncover)
export(cir_stat_PAD)
export(cir_stat_PCvM)
export(cir_stat_PRt)
export(cir_stat_Poisson)
export(cir_stat_Pycke)
export(cir_stat_Pycke_q)
export(cir_stat_Range)
export(cir_stat_Rao)
export(cir_stat_Rayleigh)
export(cir_stat_Riesz)
export(cir_stat_Rothman)
export(cir_stat_Sobolev)
export(cir_stat_Softmax)
export(cir_stat_Vacancy)
export(cir_stat_Watson)
export(cir_stat_Watson_1976)
export(con_f)
export(cutoff_locdev)
export(cutoff_wschisq)
export(d_Kolmogorov)
export(d_Sobolev)
export(d_cir_stat_Ajne)
export(d_cir_stat_Bakshaev)
export(d_cir_stat_Bingham)
export(d_cir_stat_Gine_Fn)
export(d_cir_stat_Gine_Gn)
export(d_cir_stat_Gini)
export(d_cir_stat_Gini_squared)
export(d_cir_stat_Greenwood)
export(d_cir_stat_Hermans_Rasson)
export(d_cir_stat_Hodges_Ajne)
export(d_cir_stat_Kuiper)
export(d_cir_stat_Log_gaps)
export(d_cir_stat_Max_uncover)
export(d_cir_stat_Num_uncover)
export(d_cir_stat_PAD)
export(d_cir_stat_PCvM)
export(d_cir_stat_PRt)
export(d_cir_stat_Poisson)
export(d_cir_stat_Pycke)
export(d_cir_stat_Pycke_q)
export(d_cir_stat_Range)
export(d_cir_stat_Rao)
export(d_cir_stat_Rayleigh)
export(d_cir_stat_Riesz)
export(d_cir_stat_Rothman)
export(d_cir_stat_Sobolev)
export(d_cir_stat_Softmax)
export(d_cir_stat_Stein)
export(d_cir_stat_Vacancy)
export(d_cir_stat_Watson)
export(d_cir_stat_Watson_1976)
export(d_locdev)
export(d_p_k)
export(d_proj_unif)
export(d_proj_unif_cap)
export(d_sph_stat_Ajne)
export(d_sph_stat_Bakshaev)
export(d_sph_stat_Bingham)
export(d_sph_stat_CJ12)
export(d_sph_stat_Gine_Fn)
export(d_sph_stat_Gine_Gn)
export(d_sph_stat_PAD)
export(d_sph_stat_PCvM)
export(d_sph_stat_PRt)
export(d_sph_stat_Poisson)
export(d_sph_stat_Rayleigh)
export(d_sph_stat_Rayleigh_HD)
export(d_sph_stat_Riesz)
export(d_sph_stat_Sobolev)
export(d_sph_stat_Softmax)
export(d_sph_stat_Stein)
export(d_sph_stat_Stereo)
export(d_unif_cap)
export(d_wschisq)
export(f_locdev)
export(f_locdev_Pn)
export(g_i_k)
export(int_sph_MC)
export(p_Kolmogorov)
export(p_Sobolev)
export(p_cir_stat_Ajne)
export(p_cir_stat_Bakshaev)
export(p_cir_stat_Bingham)
export(p_cir_stat_Gine_Fn)
export(p_cir_stat_Gine_Gn)
export(p_cir_stat_Gini)
export(p_cir_stat_Gini_squared)
export(p_cir_stat_Greenwood)
export(p_cir_stat_Hermans_Rasson)
export(p_cir_stat_Hodges_Ajne)
export(p_cir_stat_Hodges_Ajne2)
export(p_cir_stat_Kuiper)
export(p_cir_stat_Log_gaps)
export(p_cir_stat_Max_uncover)
export(p_cir_stat_Num_uncover)
export(p_cir_stat_PAD)
export(p_cir_stat_PCvM)
export(p_cir_stat_PRt)
export(p_cir_stat_Poisson)
export(p_cir_stat_Pycke)
export(p_cir_stat_Pycke_q)
export(p_cir_stat_Range)
export(p_cir_stat_Rao)
export(p_cir_stat_Rayleigh)
export(p_cir_stat_Riesz)
export(p_cir_stat_Rothman)
export(p_cir_stat_Sobolev)
export(p_cir_stat_Softmax)
export(p_cir_stat_Stein)
export(p_cir_stat_Vacancy)
export(p_cir_stat_Watson)
export(p_cir_stat_Watson_1976)
export(p_proj_unif)
export(p_proj_unif_cap)
export(p_sph_stat_Ajne)
export(p_sph_stat_Bakshaev)
export(p_sph_stat_Bingham)
export(p_sph_stat_CJ12)
export(p_sph_stat_Gine_Fn)
export(p_sph_stat_Gine_Gn)
export(p_sph_stat_PAD)
export(p_sph_stat_PCvM)
export(p_sph_stat_PRt)
export(p_sph_stat_Poisson)
export(p_sph_stat_Rayleigh)
export(p_sph_stat_Rayleigh_HD)
export(p_sph_stat_Riesz)
export(p_sph_stat_Sobolev)
export(p_sph_stat_Softmax)
export(p_sph_stat_Stein)
export(p_sph_stat_Stereo)
export(p_wschisq)
export(psi_Pn)
export(q_Sobolev)
export(q_proj_unif)
export(q_proj_unif_cap)
export(q_wschisq)
export(r_alt)
export(r_locdev)
export(r_proj_unif)
export(r_proj_unif_cap)
export(r_unif_cap)
export(r_unif_cir)
export(r_unif_sph)
export(r_wschisq)
export(rot_ab)
export(sph_stat_Ajne)
export(sph_stat_Bakshaev)
export(sph_stat_Bingham)
export(sph_stat_CCF09)
export(sph_stat_CJ12)
export(sph_stat_Gine_Fn)
export(sph_stat_Gine_Gn)
export(sph_stat_PAD)
export(sph_stat_PCvM)
export(sph_stat_PRt)
export(sph_stat_Poisson)
export(sph_stat_Pycke)
export(sph_stat_Rayleigh)
export(sph_stat_Rayleigh_HD)
export(sph_stat_Riesz)
export(sph_stat_Sobolev)
export(sph_stat_Softmax)
export(sph_stat_Stereo)
export(sphere_to_angles)
export(uk_to_bk)
export(uk_to_vk2)
export(unif_stat)
export(unif_stat_MC)
export(unif_stat_distr)
export(unif_test)
export(upper_tri_ind)
export(vk2_to_bk)
export(vk2_to_uk)
export(weights_dfs_Sobolev)
import(Rcpp)
importFrom(graphics,abline)
importFrom(graphics,legend)
importFrom(graphics,lines)
importFrom(graphics,par)
importFrom(graphics,segments)
importFrom(stats,approxfun)
importFrom(stats,dchisq)
importFrom(stats,dgamma)
importFrom(stats,integrate)
importFrom(stats,na.omit)
importFrom(stats,nlm)
importFrom(stats,pchisq)
importFrom(stats,pgamma)
importFrom(stats,qchisq)
importFrom(stats,qgamma)
importFrom(stats,quantile)
importFrom(stats,rbeta)
importFrom(stats,rchisq)
importFrom(stats,runif)
importFrom(stats,splinefun)
importFrom(stats,uniroot)
importFrom(utils,capture.output)
useDynLib(sphunif)
